@@ -37,8 +37,8 @@ using Poco::Nullable;
 using Poco::Tuple;
 using Poco::NamedTuple;
 
-Poco::SharedPtr<Poco::Data::Session> MySQLTest::_pSession = 0;
-Poco::SharedPtr<SQLExecutor> MySQLTest::_pExecutor = 0;
+Poco::SharedPtr<Poco::Data::Session> MySQLTest::_pSession = nullptr;
+Poco::SharedPtr<SQLExecutor> MySQLTest::_pExecutor = nullptr;
 
 //
 // Parameters for barebone-test
@@ -674,6 +674,14 @@ void MySQLTest::testNullableString()
 	assertTrue (resAddress != null);
 }
 
+void MySQLTest::testOptionalString()
+{
+	if (!_pSession) fail ("Test not available.");
+
+	recreateNullableStringTable();
+	_pExecutor->stdOptional();
+}
+
 
 void MySQLTest::testTupleWithNullable()
 {
@@ -727,6 +735,24 @@ void MySQLTest::testTupleWithNullable()
 }
 
 
+void MySQLTest::testStdTupleWithOptional()
+{
+	if (!_pSession) fail ("Test not available.");
+
+	recreateNullableStringTable();
+	_pExecutor->stdTupleWithOptional();
+}
+
+
+void MySQLTest::testSessionPoolAndUnicode()
+{
+	if (!_pSession) fail ("Test not available.");
+
+	recreateStringsTable();
+	_pExecutor->sessionPoolAndUnicode(_dbConnString);
+}
+
+
 void MySQLTest::dropTable(const std::string& tableName)
 {
 	try { *_pSession << format("DROP TABLE IF EXISTS %s", tableName), now; }
@@ -927,7 +953,7 @@ CppUnit::Test* MySQLTest::suite()
 		catch (ConnectionFailedException& ex)
 		{
 			std::cout << ex.displayText() << std::endl;
-			return 0;
+			return nullptr;
 		}
 	}
 
@@ -989,10 +1015,12 @@ CppUnit::Test* MySQLTest::suite()
 	CppUnit_addTest(pSuite, MySQLTest, testNull);
 	CppUnit_addTest(pSuite, MySQLTest, testNullableInt);
 	CppUnit_addTest(pSuite, MySQLTest, testNullableString);
+	CppUnit_addTest(pSuite, MySQLTest, testOptionalString);
 	CppUnit_addTest(pSuite, MySQLTest, testTupleWithNullable);
+	CppUnit_addTest(pSuite, MySQLTest, testStdTupleWithOptional);
 	CppUnit_addTest(pSuite, MySQLTest, testSessionTransaction);
 	CppUnit_addTest(pSuite, MySQLTest, testTransaction);
 	CppUnit_addTest(pSuite, MySQLTest, testReconnect);
-
+	CppUnit_addTest(pSuite, MySQLTest, testSessionPoolAndUnicode);
 	return pSuite;
 }
